<?php
/**
.---------------------------------------------------------------------.
|  @package: Lazy Footer
|  @version: v1.0.2 (17 February 2018)
|  @link:    http://italinux.com/addon-footer
|  @docs:    http://italinux.com/addon-footer/docs
|
|  @author: Matteo Montanari <matteo@italinux.com>
|  @link:   http://matteo-montanari.com
'---------------------------------------------------------------------'
.---------------------------------------------------------------------------.
| @copyright (c) 2018                                                       |
| ------------------------------------------------------------------------- |
| @license: Concrete5.org Marketplace Commercial Add-Ons & Themes License   |
|           http://concrete5.org/help/legal/commercial_add-on_license       |
|           or just: file://lazy_footer/LICENSE.TXT
|                                                                           |
| This program is distributed in the hope that it will be useful - WITHOUT  |
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     |
| FITNESS FOR A PARTICULAR PURPOSE.                                         |
'---------------------------------------------------------------------------'
*/
namespace Concrete\Package\LazyFooter\Block\LazyFooter;

use Concrete\Package\LazyFooter\Block\LazyFooter\Src\Utils as BlockUtils;
use Concrete\Core\Block\BlockController;
use Concrete\Core\Asset\Asset;
use Concrete\Core\Asset\AssetList;
use Concrete\Core\Editor\LinkAbstractor;
use Page;
use BlockType;
use Config;

defined('C5_EXECUTE') or die("Access Denied.");

class Controller extends BlockController
{

    protected $btTable = "btLazyFooter";

    protected static $btHandlerId = "footer";
    protected static $btViewPointIdPrefix = "wyp";

    protected $btInterfaceWidth = "1050";
    protected $btInterfaceHeight = "600";

    protected $btWrapperClass = 'ccm-ui';
    protected $btWrapperForm = 'lazy-ui';

    protected $btDefaultSet = 'lazy';

    // Style Background & Foreground Colours
    protected static $btStyleOpacity = '0.95';

    // Upload Image size in KBytes (1KB = 1024b)
    protected static $btStyleUploadImageSize = 450;

    // Upload Image Thumb Width X Height (pixels)
    protected static $btStyleUploadThumbWidth = 1190;
    protected static $btStyleUploadThumbHeight = 650;

    // Background Over Image default Opacity
    protected static $bgOverImageOpacity = 0.25;
    
    // Support for Inline Editing
    protected $btSupportsInlineEdit = false;
    protected $btSupportsInlineAdd = false;

    // Bootstrap theme Grid Support
    protected $btIgnorePageThemeGridFrameworkContainer = false;

    // Cache block's database calls
    protected $btCacheBlockRecord = true;

    // Cache block's actual view output
    protected $btCacheBlockOutput = false;

    // Serve cached version even if the result of a post request
    protected $btCacheBlockOutputOnPost = true;

    // Server cached version even if user is logged in
    protected $btCacheBlockOutputForRegisteredUsers = true;

    /**
    * When block caching is enabled and output caching is enabled for a block,
    * this is the value in seconds before cache being refreshed. Default (0) is no limit.
    */
    protected $btCacheBlockOutputLifetime = 0;

    protected static function get_btStyles()
    {
        return array(
            'bgColorRGBA' => t('Background Colour'),
            'bgColorOpacity' => t('Adjust Background Opacity'),
            'bgFID' => t('Background Image'),
            'fgColorRGB' => t('Foreground Colour'),
            'isAnimated' => t('Animation / Transition'),
            'isQuoted' => t('Wrap title with Quotes')
        );
    }

    protected static function get_btFields()
    {
        return array(
            'title' => array(
                'label' => t('Title'),
                'allowEmpty' => true,
            ),
            'subtitle' => array(
                'label' => t('Subtitle'),
                'allowEmpty' => true,
            ),
            'credits1stMessage' => array(
                'label' => t('1st Credits %s', t('wording')),
                'allowEmpty' => true,
            ),
            'credits1stName' => array(
                'label' => t('1st Credits %s', t('name')),
                'allowEmpty' => true,
            ),
            'credits1stLink' => array(
                'label' => t('1st Credits %s', t('link to name')),
                'allowEmpty' => true,
            ),
            'credits2ndMessage' => array(
                'label' => t('2nd Credits %s', t('wording')),
                'encodeEntity' => true,
                'allowEmpty' => true,
            ),
            'credits2ndName' => array(
                'label' => t('2nd Credits %s', t('name')),
                'allowEmpty' => true,
            ),
            'credits2ndLink' => array(
                'label' => t('2nd Credits %s', t('link to name')),
                'allowEmpty' => true,
            ),
        );
    }

    protected static function get_btFormExtraValues()
    {
        return array(
            'bgColorOpacityOptions' => array(
                'label' => t('Options adjust background opacity'),
                'allowEmpty' => true,
            ),
            'customTemplateName' => array(
                'label' => t('Custom template name'),
                'allowEmpty' => true,
            ),
        );
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Block info Methods
    */
    public function getBlockTypeName()
    {

        return t('Lazy Footer');
    }

    public function getBlockTypeDescription()
    {

        return t('Add Lazy Footer to your website');
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * This block Main Methods
    */
    public function getTitle() 
    {
        $cName  = 'title';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = t('a footer phrase goes here');
        
        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }
    
    public function getSubtitle() 
    {
        $cName  = 'subtitle';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = t('and subtitle here');
        
        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getIsQuoted()
    {
        $cName  = 'isQuoted';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = false;

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getCredits1stMessage()
    {
        $cName  = 'credits1stMessage';
        $config = self::$btHandlerId . '.credits.0.message';
        $dValue = t('you want to download other add-ons?');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getCredits1stName()
    {
        $cName  = 'credits1stName';
        $config = self::$btHandlerId . '.credits.0.name';
        $dValue = t('click here');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getCredits1stLink()
    {
        $cName  = 'credits1stLink';
        $config = self::$btHandlerId . '.credits.0.link';
        $dValue = 'http://matteo-montanari.com/addons';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getCredits2ndMessage()
    {
        $cName  = 'credits2ndMessage';
        $config = self::$btHandlerId . '.credits.1.message';
        $dValue = t('developed with %s by', '&hearts;');

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getCredits2ndName()
    {
        $cName  = 'credits2ndName';
        $config = self::$btHandlerId . '.credits.1.name';
        $dValue = 'Matteo';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    public function getCredits2ndLink()
    {
        $cName  = 'credits2ndLink';
        $config = self::$btHandlerId . '.credits.1.link';
        $dValue = 'http://matteo-montanari.com/its-me';

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * This Block RegisterViewAssets
    */
    public function registerViewAssets($outputContent = '')
    {
        // Import JS core Theme
        $this->requireAsset('javascript', 'jquery');

        if ($this->getIsAnimated() === true && Page::getCurrentPage()->isEditMode() == false) {
            // Import Animations Assets CSS & JS
            $this->requireAsset('jst.animate.assets');

            // Import Animations CSS & JS Configuration
            $this->requireAsset('jst.animate.conf');
        }
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * This Block View/Validate/Save Methods
    */
    public function view()
    {

        // Set style values
        $this->set('sID', $this->getSectionId());
        $this->set('viewPoint', self::getViewPointId());
        $this->set('cTemplate', $this->getCustomTemplateName());
        $this->set('cFgColorClass', $this->getCustomFgColorClassName());

        // Set main values
        // 1st Credits CTA button
        $this->set('credits1st', array('text' => $this->getCredits1stMessage(),
                                       'name' => $this->getCredits1stName(),
                                      'class' => BlockUtils::getClassByLink($this->getCredits1stLink(), null),
                                       'link' => $this->getCredits1stLink(),
                                     'target' => BlockUtils::getTargetByLink($this->getCredits1stLink())));

        // 1st Credits CTA button
        $this->set('credits2nd', array('text' => $this->getCredits2ndMessage(),
                                       'name' => $this->getCredits2ndName(),
                                      'class' => BlockUtils::getClassByLink($this->getCredits2ndLink(), null),
                                       'link' => $this->getCredits2ndLink(),
                                     'target' => BlockUtils::getTargetByLink($this->getCredits2ndLink())));

        // Import Custom Css3 inline
        $this->set('cStyle', $this->getCustomStyle());
    }

    public function on_start()
    {
        $al = AssetList::getInstance();

        $pf = Array(
            'position' => Asset::ASSET_POSITION_FOOTER,
            'minify' => true,
            'combine' => true
        );

        $ph = Array(
            'position' => Asset::ASSET_POSITION_HEADER,
            'minify' => true,
            'combine' => true
        );

        $cf = Array(
            'position' => Asset::ASSET_POSITION_FOOTER,
            'minify' => false,
            'combine' => false
        );

        /** - - - - - - - - - - - - - - - - - - - - - - - - -
         * Register JS / CSS Animate for this Block
         */
        $al->register('javascript', 'jt.jquery.waypoints', 'blocks/lazy_footer/jscript/min/jquery.waypoints.min.js', $pf, 'lazy_footer');

        // Register Assets Animate
        $al->register('javascript', 'animate-lib', 'blocks/lazy_footer/jscript/min/jquery.lazy.animate.min.js', $pf, 'lazy_footer');

        $al->register('css', 'style.animate', 'blocks/lazy_footer/style/animate.min.css', $ph, 'lazy_footer');
        $al->register('css', 'style.animate.delay', 'blocks/lazy_footer/style/animate.delay.min.css', $ph, 'lazy_footer');

        $al->registerGroup(
            'jst.animate.assets', array(
                array(
                    'javascript',
                    'jt.jquery.waypoints'
               ),
               array(
                   'javascript',
                   'animate-lib'
               ),
               array(
                   'css',
                   'style.animate'
               ),
               array(
                   'css',
                   'style.animate.delay'
               ),
            )
        );

        // Register Assets Animate Configuration
        $al->register('javascript', $this->getJSelectorId() . '.animate-conf', 'blocks/lazy_footer/jscript/lazy-animate.conf.js', $pf, 'lazy_footer');
        $al->register('javascript-inline', $this->getJSelectorId() . '.animate-init',  '$("section#' . $this->getSectionId()  . '").lazyAnimateInit();', $cf, 'lazy_footer');

        $al->registerGroup(
            'jst.animate.conf', array(
               array(
                   'javascript',
                   $this->getJSelectorId() . '.animate-conf'
               ),
               array(
                   'javascript-inline',
                   $this->getJSelectorId() . '.animate-init'
               ),
            )
        );
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Custom inline Style main
    * Stream
    */
    protected function getCustomStyle()
    {
        $o=null;

        if (BlockUtils::isValidColor($this->bgColorRGBA) ||
            BlockUtils::isValidImage($this->getBgFID()) ||
            BlockUtils::isValidColor($this->fgColorRGB) ||
            $this->isCustomOverImageOpacity($this->bgColorOpacity)) {

            ob_start();
            ?>

            <style>
              <?php
                if (BlockUtils::isValidColor($this->bgColorRGBA) ||
                    $this->isCustomOverImageOpacity($this->bgColorOpacity)) {
                ?>
                  footer<?php echo $this->getStyleSelector()?>.over-image::before {
                    <?php
                      if (BlockUtils::isValidColor($this->bgColorRGBA)) { ?>
                          background-color: <?php echo $this->getOverImageBgColor()?> !important;
                    <?php } ?>
                    <?php
                      if ($this->isCustomOverImageOpacity($this->bgColorOpacity)) { ?>
                          opacity: <?php echo $this->bgColorOpacity?> !important;
                    <?php } ?>
                  }
              <?php } ?>

              <?php
                if (BlockUtils::isValidImage($this->getBgFID()) ||
                    BlockUtils::isValidColor($this->bgColorRGBA) ||
                    BlockUtils::isValidColor($this->fgColorRGB)) {
                ?>
                  footer<?php echo $this->getStyleSelector()?>.over-image {
                    <?php
                      if (BlockUtils::isValidImage($this->getBgFID())) { ?>
                          background-image: url('<?php echo $this->getCustomStyleImagePath()?>') !important;
                    <?php } ?>

                    <?php
                      if (BlockUtils::isValidColor($this->bgColorRGBA)) { ?>
                          background-color: <?php echo $this->bgColorRGBA?> !important;
                    <?php } ?>

                    <?php
                      if (BlockUtils::isValidColor($this->fgColorRGB)) { ?>
                          color: <?php echo $this->fgColorRGB ?> !important;
                    <?php } ?>
                  }
               <?php } ?>
            </style>

            <?php
            $o = ob_get_contents();

            ob_end_clean();

            $o = BlockUtils::getCustomStyleSanitised($o);
        }

        return $o;
    }
 
    public function validate($args)
    {
        $e = BlockUtils::getThisApp()->make('error');

        // Validate Styles
        foreach (self::get_btStyles() as $key => $value) {

            switch ($key) {
            case (lcfirst(substr($key, -3)) == 'fID'):

                if (!empty($args[$key])) {
                    $f = BlockUtils::getFileObject($args[$key]);

                    if (is_object($f)) {
                        switch ($f->getMimeType()) {
                        case 'image/jpeg':
                        case 'image/jpg':
                        case 'image/png':
                            break;
                        default:
                            $e->add(t('File type required: JPG or PNG'));
                            break;
                        }

                        switch (strtolower($f->getExtension())) {
                        case 'jpeg':
                        case 'jpg':
                        case 'png':
                            break;
                        default:
                            $e->add(t('File extension required: JPG or PNG'));
                            break;
                        }

                        $uploadImageSize = BlockUtils::getUploadImageSize(self::$btStyleUploadImageSize);

                        if ($f->getFullSize() > $uploadImageSize) {
                            $e->add(t('File size exceeded: Max %s', BlockUtils::getHumanReadUploadImageSize($uploadImageSize)));
                        }

                        if ($f->isError()) {
                            $e->add(t('File image is invalid'));
                        }
                    } else {
                        $e->add(t('File image is invalid'));
                    }
                }
                break;
            }
        }

        // Validate Fields
        foreach (self::get_btFields() as $key => $value) {

            // Field is Empty
            if (isset($value['allowEmpty']) && ($value['allowEmpty'] === false)) {

                if (empty($args[$key])) {
                    $e->add(t('Cannot be empty: %s', $value['label']));
                }
            }
        }

        return $e;
    }
 
    public function save($args)
    {

        // Update custom Styles for all pages 
        foreach (self::get_btStyles() as $key => $value) {

            switch ($key) {
            case (lcfirst(substr($key, -3)) == 'fID'):
                if (empty($args[$key])) {
                    $args[$key] = 0;
                }
                break;
            case 'bgColorRGBA':
                if (empty($args[$key])) {
                    $args[$key] = 'transparent';
                }
                break;
            case 'fgColorRGB':
                if (empty($args[$key])) {
                    $args[$key] = 'transparent';
                }
                break;
            }
        }

        // Sanitize input for DB
        foreach (self::get_btFields() as $key => $value) {

            // Encode HTML Entities
            if (isset($value['encodeEntity']) && ($value['encodeEntity'] === true)) {
                $args[$key] = htmlentities($args[$key]);
            }

            // Prevent Default Value from setting
            if (empty($args[$key])) {
                $args[$key] = ' ';
            }
        }

        parent::save($args);
    }
 
    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Block Handlers Design Methods
    */
    protected static function getViewPointId()
    {
        return self::$btViewPointIdPrefix . '-' . self::$btHandlerId;
    }

    protected function getSectionId()
    {
        return 's' . $this->bID;
    }

    protected function getStyleSelector()
    {
        return '#' . $this->getSectionId() . '.' . self::$btHandlerId;
    }

    protected function getJSelectorId()
    {
        return $this->getSectionId() . '.' . self::$btHandlerId;
    }

    protected function getCustomTemplateName()
    {
        $tName = 'no_template';
        $block = $this->getBlockObject();

        if (is_object($block)) {
          $tName = ($block->getBlockFilename() == true ? $block->getBlockFilename() : $tName);
        } 

        return $tName;
    }

    protected function getCustomFgColorClassName()
    {
        return (BlockUtils::isValidColor($this->fgColorRGB) ? 'cfg-color' : null);
    }

    public static function getBlockHandle()
    {
        return strtolower(basename(dirname(__FILE__)));
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Block - Foreground Custom Images PATHs
    *
    * @return image full Path (file system)
    */
    protected function getBlockForegroundImageURL($fID, $w = null, $h = null)
    {
        // set thumbnail width
        $width = ($w == false) ? self::$btCustomImageThumbWidth : $w;

        // set thumbnail height
        $height = ($h == false) ? self::$btCustomImageThumbHeight : $h;

        // get source thumbnail image
        return (is_object($fID) ? BlockUtils::getThisApp()->make('helper/image')->getThumbnail($fID, $width, $height, true)->src : null);
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template & Block - Default & Background Images URLs
    *
    * @return image URL (for HTML src attribute)
    */
    protected function getBlockDefaultImageURL()
    {
        $image = $this->getDefaultImageFullSrc();

        return $image['url'];
    }

    protected function getBlockBackgroundImageURL()
    {
        $image = $this->getBackgroundImageFullSrc();

        return $image['url'];
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template & Block - Default & Background Images PATHs
    *
    * @return image full Path (file system)
    */
    protected function getBlockDefaultImagePath()
    {
        $image = $this->getDefaultImageFullSrc();

        return $image['path'];
    }

    protected function getBlockBackgroundImagePath()
    {
        $image = $this->getBackgroundImageFullSrc();

        return $image['path'];
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template & Block - Default Images Width & Height
    *
    * @return image full Path (file system)
    */
    protected function getBlockDefaultImageWidth()
    {
        $size = @getimagesize($this->getBlockDefaultImagePath());

        // get default image width
        return $size[0];
    }

    protected function getBlockDefaultImageHeight()
    {
        $size = @getimagesize($this->getBlockDefaultImagePath());

        // get default image height
        return $size[1];
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template & Block - Paths & URLs 
    *
    * @param true / false
    * @return image base Path and URL (Template / Block)
    */
    protected function getBlockImagesBaseURL($tPath=false)
    {
        return $this->getBlockAssetsURL() . ($tPath !== false ? $this->getBlockTemplateRelativePath() : null) . '/images';
    }

    protected function getBlockImagesBasePath($tPath=false)
    {
        return __DIR__ . ($tPath !== false ? $this->getBlockTemplateRelativePath() : null) . '/images';
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Template - Paths & URLs 
    *
    * @return image base Path and URL for a Template
    */
    protected function getBlockImagesBaseTemplateURL()
    {
        return $this->getBlockImagesBaseURL(true);
    }

    protected function getBlockImagesBaseTemplatePath()
    {
        return $this->getBlockImagesBasePath(true);
    }

    /** Get Template relative Path */
    protected function getBlockTemplateRelativePath()
    {
        return ($this->getCustomTemplateName() == false ? null : '/templates/' . $this->getCustomTemplateName());
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Block functional Methods
    */
    private function getBackgroundImageFullSrc()
    {
        return $this->getDefaultImageFullSrc('background');
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * get Template or Block images Paths & URLs 
    * IMPORTANT: it uses inheritance
    * Returns image from Template or Block folders
    *
    * @param default / background
    * @return array (path, url)
    */
    private function getDefaultImageFullSrc($file='default')
    {
        $image = null;
        $ext = array('png', 'jpg');

        // check if Template Image folder has images 
        if (is_dir($this->getBlockImagesBaseTemplatePath())) {
            foreach ($ext as $value) {
                $imgPath = $this->getBlockImagesBaseTemplatePath() . '/' . $file . '.' . $value;
                
                if (is_file($imgPath)) {
                    $image = array('path' => $imgPath,
                                   'url' => $this->getBlockImagesBaseTemplateURL() . '/' . $file . '.' . $value);
                } 
            }
        } 

        // otherwise use images within Block 
        if ($image == false) {
            foreach ($ext as $value) {
                $imgPath = $this->getBlockImagesBasePath() . '/' . $file . '.' . $value;
                
                if (is_file($imgPath)) {
                    $image = array('path' => $imgPath,
                                   'url' => $this->getBlockImagesBaseURL() . '/' . $file . '.' . $value);
                } 
            }
        }

        return (array) $image;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Custom inline Style Methods
    * Background inline Styles Methods
    */
    public function getBgFID()
    {
        if ($this->bgFID > 0) {
            $fObj = BlockUtils::getFileObject($this->bgFID);
        }

        return (isset($fObj) && is_object($fObj)) ? $fObj : null;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Check if Animation is enabled (ideal for JS variables)
    */
    public function getIsAnimationEnabled()
    {        
        return ($this->getIsAnimated() === true && Page::getCurrentPage()->isEditMode() == false) ? 'true' : 'false';
    }

    public function getIsEditMode()
    {
        return (Page::getCurrentPage()->isEditMode() == false) ? 0 : 1;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Custom Animation / Transition
    */
    public function getIsAnimated()
    {
        $cName  = 'isAnimated';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = true;

        return filter_var(BlockUtils::getDefaultValue($config, $dValue, $this->{$cName}), FILTER_VALIDATE_BOOLEAN);
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Background Opacity Over Image
    */
    public function getBgColorOpacity()
    {
        $cName  = 'bgColorOpacity';
        $config = self::$btHandlerId . '.' . $cName;
        $dValue = self::$bgOverImageOpacity;

        return BlockUtils::getDefaultValue($config, $dValue, $this->{$cName});
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Background Opacity Over Image Options
    */
    public function getBgColorOpacityOptions()
    {
        return array(
            '-60%' => 0.25,
            '-40%' => 0.5,
            '-20%' => 0.75,
            'default' => 1
        );
    }

    protected function getCustomStyleImagePath()
    {
        return BlockUtils::getThisApp()->make('helper/image')->getThumbnail($this->getBgFID(), self::$btStyleUploadThumbWidth, self::$btStyleUploadThumbHeight, false)->src;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Custom inline Style Methods
    */
    protected function isCustomOverImageOpacity($value)
    {
        return ($value == true && (self::$bgOverImageOpacity != $this->getBgColorOpacity())) === true ? true : false;
    }

    protected function getOverImageBgColor()
    {
        return $this->bgColorRGBA;
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Form Overlay (Edit/Add) Methods
    */
    public function getBlockAssetsURL()
    {
        $bt = BlockType::getByHandle(self::getBlockHandle());
        $bPath = BlockUtils::getThisApp()->make('helper/concrete/urls')->getBlockTypeAssetsURL($bt);

        return $bPath;
    }

    protected function addLocalAssets($path, $type = 'css')
    {
        $this->addHeaderItem(BlockUtils::getThisApp()->make('helper/html')->{$type}($this->getBlockAssetsURL() . $path));
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Add Form (Window Overlay) - default Values
    */
    protected function addFormDefaultValues()
    {
        // Retrieve defaults Values
        if (method_exists(__CLASS__, 'get_btFields')) {

            foreach (self::get_btFields() as $key => $value) {

                if (method_exists($this, 'get' . ucfirst($key))) {

                    $o = $this->{'get' . ucfirst($key)}();

                    // Encode HTML Entities
                    if (isset($value['encodeEntity']) && ($value['encodeEntity'] === true)) {
                        $o = html_entity_decode($o);
                    }

                    $o = is_array($o) ? $o : trim($o);

                    $this->set($key, $o);
                }
            }
        }
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Add Form (Window Overlay) - extra Values
    */
    protected function addFormExtraValues()
    {
        // Retrieve defaults Values
        if (method_exists(__CLASS__, 'get_btFields')) {
            foreach (array_keys(self::get_btStyles()) as $key) {
                if (method_exists($this, 'get' . ucfirst($key))) {
                    $this->set($key, $this->{'get' . ucfirst($key)}());
                }
            }
        }

        // Retrieve extra Values
        if (method_exists(__CLASS__, 'get_btFormExtraValues')) {
            foreach (array_keys(self::get_btFormExtraValues()) as $key) {
                if (method_exists($this, 'get' . ucfirst($key))) {
                    $this->set($key, $this->{'get' . ucfirst($key)}());
                }
            }
        }
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Add Form (Window Overlay)
    */
    protected function add()
    {

        // Custom Styles Palettes
        $this->set('color', BlockUtils::getThisApp()->make('helper/form/color'));
        $this->set('asset', BlockUtils::getThisApp()->make('helper/concrete/asset_library'));

        $this->set('fgColorPalette', BlockUtils::getFgColorPalette(false, true));
        $this->set('bgColorPalette', BlockUtils::getBgColorPalette(true, true, self::$btStyleOpacity));

        $this->set('btWrapperForm', $this->btWrapperForm);

        // User Interface
        $this->set('hUI', BlockUtils::getThisApp()->make('helper/concrete/ui'));

        $this->addFormDefaultValues();
        $this->addFormExtraValues();

        // Add Assets to Window Overlay
        $this->addLocalAssets('/css/tools/lazy-global-ui.css', 'css');
    }

    /** - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Edit Form (Window Overlay)
    */
    protected function edit() 
    {
        $this->add();
    }
}
